"use strict";
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const _1 = __importDefault(require("."));
// TODO(ariels): Change db-client to allow starting a client late,
//     *without* environment variables.
const db_1 = require("@reshuffle/db/dist/db");
const fs_1 = require("fs");
const rmfr_1 = __importDefault(require("rmfr"));
const os_1 = require("os");
const util_1 = require("util");
const path = __importStar(require("path"));
const leveldb_server_1 = require("@reshuffle/leveldb-server");
const interfaces_koa_server_1 = require("@reshuffle/interfaces-koa-server");
const koa_1 = __importDefault(require("koa"));
const koa_router_1 = __importDefault(require("koa-router"));
const test = ava_1.default;
const app = new koa_1.default();
const router = new koa_router_1.default();
app.use(router.routes());
app.use(router.allowedMethods());
const server = app.listen(0, '127.0.0.1');
const port = new Promise((res) => {
    server.once('listening', () => {
        res(server.address().port);
    });
});
let count = 0;
test.beforeEach(async (t) => {
    const dbDir = await util_1.promisify(fs_1.mkdtemp)(path.join(os_1.tmpdir(), 'test-state-'), 'utf8');
    const handler = new leveldb_server_1.Handler(`${dbDir}/root.db`, undefined, (err) => t.fail(err && err.toString()));
    const dbRouter = new interfaces_koa_server_1.DBRouter(handler);
    const title = `${++count}`;
    router.use(`/${title}`, dbRouter.koaRouter.routes(), dbRouter.koaRouter.allowedMethods());
    const ctx = {
        appId: `testing:${title}`,
        appEnv: 'default',
        collection: 'test',
        auth: {},
    };
    const db = new db_1.DB(`http://localhost:${await port}/${title}`, ctx, { timeoutMs: 1000 });
    t.context = {
        db,
        dbDir,
    };
});
test.afterEach(async (t) => {
    await rmfr_1.default(t.context.dbDir);
});
async function iteratorToArray(it) {
    var e_1, _a;
    const res = [];
    try {
        for (var it_1 = __asyncValues(it), it_1_1; it_1_1 = await it_1.next(), !it_1_1.done;) {
            const next = it_1_1.value;
            res.push(next);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (it_1_1 && !it_1_1.done && (_a = it_1.return)) await _a.call(it_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return res;
}
test('iterateFind over nothing returns nothing', async (t) => {
    const { db } = t.context;
    await db.create('b', 2);
    const filter = db_1.Q.key.eq('a');
    const result = await iteratorToArray(_1.default(db_1.Q.filter(filter), undefined, db));
    t.deepEqual(result, []);
});
test('iterateFind over single chunk returns that chunk', async (t) => {
    const { db } = t.context;
    await db.create('a', 1);
    await db.create('b', 2);
    const filter = db_1.Q.key.gte('a');
    const result = await iteratorToArray(_1.default(db_1.Q.filter(filter), undefined, db));
    t.deepEqual(result, [{ key: 'a', value: 1 }, { key: 'b', value: 2 }]);
});
test('iterateFind over multiple chunks', async (t) => {
    const { db } = t.context;
    await db.create('a', 1);
    await db.create('b', 2);
    await db.create('c', 3);
    const filter = db_1.Q.key.startsWith('');
    const result = await iteratorToArray(_1.default(db_1.Q.filter(filter).orderBy(db_1.Q.key), { chunkSize: 2 }, db));
    t.deepEqual(result, [{ key: 'a', value: 1 }, { key: 'b', value: 2 }, { key: 'c', value: 3 }]);
});
test.skip('iterateFind retrieves a snapshot', async (t) => {
    const { db } = t.context;
    await db.create('a', 1);
    await db.create('b', 2);
    const filter = db_1.Q.key.startsWith('');
    const finder = _1.default(db_1.Q.filter(filter).orderBy(db_1.Q.key), { chunkSize: 1 }, db);
    t.deepEqual((await finder.next()).value, { key: 'a', value: 1 });
    await db.create('c', 3);
    const result = await iteratorToArray(finder);
    t.deepEqual(result, [{ key: 'b', value: 2 }]);
});
test('iterateFind works for DBs of non-trivial size', async (t) => {
    const size = 100;
    const { db } = t.context;
    const pairs = Array.from({ length: size }).map((_, index) => {
        const i = index.toString().padStart(6, '0');
        return { key: `key:${i}`, value: `value:${i}` };
    });
    await Promise.all(pairs.map(({ key, value }) => db.create(key, value)));
    const filter = db_1.Q.key.startsWith('');
    const result = await iteratorToArray(_1.default(db_1.Q.filter(filter).orderBy(db_1.Q.key), { chunkSize: 37 }, db));
    t.deepEqual(result, pairs);
});
//# sourceMappingURL=iterator.spec.js.map